@@ -1,4 +1,6 @@
-xmlize = {}
+from typing import Dict, Optional, Any, Union
+
+xmlize: Dict[str, str] = {}
 xmlize[">"] = "&gt;"
 xmlize["<"] = "&lt;"
 xmlize["&"] = "&amp;"
@@ -16,20 +18,21 @@ class XMLBase:
     XMLBase is the base class for all exportable objects in drawpyo. This class defines a few useful properties that drawpyo needs to use to generate a Draw.io file.
     """
 
-    def __init__(self, **kwargs):
-        self._id = kwargs.get("id", id(self))
-        self.xml_class = kwargs.get("xml_class", "xml_tag")
+    def __init__(self, **kwargs: Any) -> None:
+        self._id: Union[int, str] = kwargs.get("id", id(self))
+        self.xml_class: str = kwargs.get("xml_class", "xml_tag")
 
         # There's only one situation where XMLBase is called directly: to
         # create the two empty mxCell objects at the beginning of every
         # Draw.io diagram. The following declarations should be overwritten
         # in every other use case.
-        self.xml_parent = kwargs.get("xml_parent", None)
+        self.xml_parent: Optional[Any] = kwargs.get("xml_parent", None)
 
-        self.tag = kwargs.get("tag", None)
+        self.tag: Optional[str] = kwargs.get("tag", None)
+        self.tooltip: Optional[str] = kwargs.get("tooltip", None)
 
     @property
-    def id(self):
+    def id(self) -> Union[int, str]:
         """
         id is a unique identifier. Draw.io generated diagrams use an ID many more characters but the app isn't picky when parsing so drawpyo just uses Python's built-in id() function as it guarantees unique identifiers.
 
@@ -39,7 +42,7 @@ class XMLBase:
         return self._id
 
     @property
-    def attributes(self):
+    def attributes(self) -> Dict[str, Any]:
         """
         The most basic attributes of a Draw.io object. Extended by subclasses.
 
@@ -53,11 +56,13 @@ class XMLBase:
     ###########################################################
 
     @property
-    def xml_open_tag(self):
+    def xml_open_tag(self) -> str:
         """
         The open tag contains the name of the object but also the attribute tags. This property function concatenates all the attributes in the class along with the opening and closing angle brackets and returns them as a string.
 
-        When the "tag" attribute tag is provided, a tags attribute is applied to the object. This allows for selecting, hiding, or displaying multiple elements in the diagram. When using tags, the open_tag value and id are shifted to the <UserObject> tag.
+        When the "tag" attribute tag is provided, a tags attribute is applied to the object. This allows for selecting, hiding, or displaying multiple elements in the diagram.
+        When the "tooltip" attribute tag is provided, a tooltip attribute is applied to the object. This allows for extra information to be displayed when an element is hovered over in the diagram.
+        When using tags or tooltips, the open_tag value and id are shifted to the <UserObject> tag.
 
 
         Example:
@@ -66,10 +71,14 @@ class XMLBase:
         Returns:
             str: The opening tag of the object with all the attributes.
         """
-        if self.tag:
-            open_user_object_tag = (
-                f'<UserObject label="{self.value}" tags="{self.tag}" id="{self.id}">'
-            )
+        if self.tag or self.tooltip:
+            open_user_object_tag = f'<UserObject label="{self.value}" id="{self.id}"'
+            if self.tag:
+                open_user_object_tag += f' tags="{self.tag}"'
+            if self.tooltip:
+                open_user_object_tag += f' tooltip="{self.xml_ify(self.tooltip)}"'
+            open_user_object_tag += ">"
+
             open_tag = "<" + self.xml_class
             for att, value in self.attributes.items():
                 if att == "id" or att == "value":
@@ -86,7 +95,7 @@ class XMLBase:
         return open_tag + ">"
 
     @property
-    def xml_close_tag(self):
+    def xml_close_tag(self) -> str:
         """
         The closing tag contains the name of the object wrapped in angle brackets.
 
@@ -96,12 +105,12 @@ class XMLBase:
         Returns:
             str: The closing tag of the object with all the attributes.
         """
-        if self.tag:
+        if self.tag or self.tooltip:
             return "</{0}>\n</UserObject>".format(self.xml_class)
         return "</{0}>".format(self.xml_class)
 
     @property
-    def xml(self):
+    def xml(self) -> str:
         """
         All drawpyo exportable classes contain an xml property that returns the formatted string of their XML output.
 
@@ -115,11 +124,11 @@ class XMLBase:
         """
         return self.xml_open_tag[:-1] + " />"
 
-    def xml_ify(self, parameter_string):
+    def xml_ify(self, parameter_string: str) -> str:
         return self.translate_txt(parameter_string, xmlize)
 
     @staticmethod
-    def translate_txt(string, replacement_dict):
+    def translate_txt(string: str, replacement_dict: Dict[str, str]) -> str:
         new_str = ""
         for char in string:
             if char in replacement_dict:
